#include "Board.h"
#include <qpainter.h>
#include <qimage.h>
#include <qapplication.h>
#include <qcursor.h>
#include <qwmatrix.h> 
#include <qmessagebox.h> 
#include <qapplication.h>
#include "ImageRepository.h"

//----------------------------------------------------------------



#define BOXSIZE 32
#define BORDER 4
#define BOARDSIZE (8*BOXSIZE+BORDER)
#define BMOFFSET 5


const char Board::flip[9] = {EMPTY, WHITE, BLACK, 0, 0, WHITEKING, BLACKKING, 0, BLUE};
const char Board::_usermoves[] ="B1D1F1H1A2C2E2G2??B3D3F3H3A4C4E4G4??B5D5F5H5A6C6E6G6??B7D7F7H7A8C8E8G8??";
const int Board::offset[9][4] = 
{	// legal move directions
		{0,0,0,0},
		{-5,-4,0,0},
		{4,5,0,0},
		{0,0,0,0},
		{0,0,0,0},
		{4,5,-4,-5},
		{4,5,-4,-5},
		{0,0,0,0},
		{0,0,0,0}
};

Board::Board(ImageRepository *img,int w, int h,QWidget* parent, const char* name, WFlags f ):QWidget(parent,name,f)
{
	seed = time(0);
	init();
	images=img;
	setBackgroundMode(NoBackground);
	animationList.setAutoDelete(true);
	connect( &timer, SIGNAL(timeout()),this, SLOT(synchTimeout()));

}

void Board::resizeEvent ( QResizeEvent * )
{
	boxsize=(width())/8;
	back.resize(width(),height());
	updateBoard();
	repaint();
	
}

void Board::createHighlightedImage(QImage &img)
{
   int x,y;

   for(y=0;y<img.height();y++)
   {
      for(x=0;x<img.width();x++)
      {
         QRgb s=img.pixel(x,y);
         QColor clr(s);
         QRgb d=clr.dark(150).rgb();
         img.setPixel(x,y,qRgba(qRed(d),qGreen(d),qBlue(d),qAlpha(s)));         
      }
   }

}

int Board::squarex(int i) 
{
	return (usermoves(i,1)-'A')*boxsize;
}
int Board::squarey(int i) 
{
	return (usermoves(i,2)-'1')*boxsize;
}


void Board::updateBoard(QPainter *p)
{
	bool heapP=false;

	if(!p)
	{
		p=new QPainter(&back);
		heapP=true;
	}
	
	//QColor black(0,0,0);
	//QColor dark(128,128,128);
	//QColor light(192,192,192);
	
	//make_bitmaps();
	p->drawRect(0,0,width()-1,height()-1);
	int x; 
	for (x=0; x<8; x++) 
	{
		for (int y=0; y<8; y++) 
		{
			if (!((x^y)&1)) 
				p->drawImage(x*boxsize, y*boxsize,*images->findImage("light_square"));
			else
				p->drawImage(x*boxsize, y*boxsize,*images->findImage("dark_square"));
		}
	}
	
	if(playing)
	{
		if (showlegal) 
		{
			node* n;
			p->setPen(QColor(0,0,0));
			p->setBrush(QColor(92,92,92));
			for (n = root->son; n; n = showlegal==2 ? n->son : n->brother) 
			{
				int x1 = squarex(n->from)+boxsize/2;
				int y1 = squarey(n->from)+boxsize/2;
				int x2 = squarex(n->to)+boxsize/2;
				int y2 = squarey(n->to)+boxsize/2;
				p->drawLine(x1,y1,x2,y2);
				p->fillRect(x2-2,y2-2,4,4,QColor(0,0,0));
			}
		}
		for (int j = 5; j < 40; j++) 
		{		 		
			if(j!=animating)
				draw_piece(p,j,b[j], squarex(j), squarey(j));		
		}	
	}

	if(heapP)
		delete p;
	
}

void Board::paintEvent ( QPaintEvent * ) 
{
	QPainter p(this);
	p.drawPixmap(0,0,back);
	
}



void Board::synchTimeout()
{

	drawNew=true;
}

void Board::animate(node* move, int backwards) 
{

	if (!move) 
		return;
	/*
	int f = move->from;
	int t = move->to;
	if (backwards) 
	{
		int x = f; f = t; t = x;
	}
	int x1 = squarex(f);
	int y1 = squarey(f);
	int x2 = squarex(t);
	int y2 = squarey(t);
	const int STEPS=4;
	animating=move->from;

	timer.start(1425);
	QPainter p(&back);	
	
	updateBoard(&p);
	for (int j=0; j<STEPS; j++) 
	{
		int x = x1+(x2-x1)*j/STEPS;
		int y = y1+(y2-y1)*j/STEPS;
		updateBoard(&p);
		draw_piece(&p,move->from,b[move->from],x,y);
		paintEvent(0);//repaint();
		drawNew=false;
		while(!drawNew)
		{
			qApp->processEvents ();
			if(aquit)
				exit(0);
		}		
	}
	animating=-1;
	timer.stop();
	*/
	updateBoard();
	
}
void Board::computer_move(int help) 
{
	
	if (!playing) 
		return;

	emit computerMove();
	QApplication::setOverrideCursor ( WaitCursor ) ;

	busy = 1; 
	abortflag = 0;
	
	node* move = calcmove(root);
	busy = 0;
	
	if (move) 
	{

		if (!help && move->value <= -30000) 
		{
			playing = autoplay = 0;
			QApplication::restoreOverrideCursor();
			updateBoard();
			repaint();
			emit computerResigned();
			return;
		}	
		animate(move,0);
		domove(move);
		
	}
	expandnode(root);
	QApplication::restoreOverrideCursor();

	if (!root->son) 
	{
		updateBoard();
		repaint();
		playing = autoplay = 0;
		emit gameOver(root->who ? BLACK: WHITE);			
	}
	 
		
	
}

void Board::mousePressEvent ( QMouseEvent *e )
{
	if(playing && !autoplay)
	{
		if(selectedPiece!=-1)
		{
			if(movePiece(e->x(),e->y()))
			{
				
				selectedPiece=-1;
				updateBoard();
				repaint();				
				computer_move(0);
				if(playing)
				{
					emit playerMove();
					emit undoAvailable(true);
				}
				updateBoard();
				repaint();
				return;
			}			
		}
		
		node* nd=selectPiece(e->x(),e->y());
		if(nd)
		{
			selectedPiece=nd->from;
			updateBoard();
			repaint();
		}
		
	}
}
bool Board::movePiece(int x, int y)
{
	// find the closest legal move he dropped it on:

	node *nd=0;
	node *t;

	for (t = root->son; t; t = t->brother)  
	{
		if (t->from==selectedPiece)
		{
			int nx = squarex(t->to);
			int ny = squarey(t->to);
			
			QRect r(nx,ny,boxsize,boxsize);
			
			if (r.contains(x,y)) 
			{			
				nd=t;
				break;
				return nd;
			}

			/*int d1 = (x-squarex(t->to));
			int d = d1*d1;
			d1 = (y-squarey(t->to));
			d += d1*d1;
			if (d < dist) 
			{
				dist = d; 
				nd=t;
			}*/
		}
	}
	if (!nd) 
	{
		//drop_piece(erase_this); 
		return false;
	} // none found
	//animate(nd,0); 
	domove(nd);
	
	//if (showlegal) {showlegal = 0; redraw();}
	//if (n->jump) 
///		redraw();

	
	return true;
}


Board::node* Board::selectPiece(int x, int y)
{
	node* nd=0;
	expandnode(root);
	for (nd = root->son; nd; nd = nd->brother) 
	{
		int nx = squarex(nd->from);
		int ny = squarey(nd->from);

		QRect r(nx,ny,boxsize,boxsize);

		if (r.contains(x,y)) 
		{			
			return nd;
		}
		
	}
	return 0;
}

void Board::draw_piece(QPainter *p,int num,int which, int x, int y) 
{
	/*if (!fl_not_clipped(x,y,ISIZE,ISIZE)) 
		return;*/

	QString name;

	switch (which) 
	{
		case BLACK: 
			name="blue_pion";
			break;
		case WHITE: 
			name="red_pion";
			break;
		case BLACKKING:
			name="blue_queen";			
			break;
		case WHITEKING: 
			name="red_queen";	
			break;
		default: 
			return;
	}
	if(num==selectedPiece)
		name.append("h");	
	p->drawImage(x,y,*images->findImage(name));	

}


void Board::init()
{
	aquit=false;

	//maxevaluate=2500;		// max number of moves to examine on a turn
	maxevaluate=maxnodes=10000;
	//maxnodes = 2500;		// maximum number of nodes in search tree
	maxply = 20;		// maximum depth to look ahead
	forcejumps = 1;		// is forced jumps rule in effect?
	
	// scoring parameters: (all divided by 5 from original code)
	// some signs seem to be backwards, marked them with (-) in comment
	spiece = 800;		// value of a piece
	sking = 1200;		// value of a king
	sadvan = 160;		// value of mypieces/theirpieces-1	
	sallpin = 80;		// mobil == 0
	sdeny = 10;		// moves enemy can make that will be jumped
	spin = 32;		// enemy pieces that have no move except jumped
	sthreat = -10;	// enemy pieces we can jump if not moved (-)
	sgrad = 1;		// score of piece positions
	sback = 10;		// back row occupied so enemy can't make king
	smoc2 = 200;		// more mobility, more center
	smoc3 = -8;		// less mobility, less center
	smoc4 = -80;		// more mobility, less center
	smode2 = -14;		// less mobility, less denied
	smode3 = -40;		// more mobility, more denied (-)
	sdemmo = -20;		// more denied, more moves (-)
	scent = 10;		// pieces in center
	skcent = 100;		// kings in center
	depthpenalty=4;	// guess
	noise=2;		// values less or eq to this apart are eq
	undoroot=root=freelist=0;
	playing=false;
	showlegal=0;
	autoplay=0;
	animating=-1;

}

void Board::newgame(int color) 
{	
	int n;
	selectedPiece=-1;
	for (n=0; n<5; n++) 
		b[n] = BLUE;
	for (n=5; n<18; n++) 
		b[n] = WHITE;
	for (n=18; n<27; n++) 
		b[n] = EMPTY;
	for (n=27; n<40; n++) 
		b[n] = BLACK;
	for (n=40; n<45; n++) 
		b[n] = BLUE;
	
	b[13] = b[22] = b[31] = BLUE;
	
	centralsquares[15] = centralsquares[16] =
		centralsquares[19] = centralsquares[20] =
		centralsquares[24] = centralsquares[25] =
		centralsquares[28] = centralsquares[29] = 1;
	
	// set up initial search tree:
	nextjump = 0;
	killnode(undoroot);
	undoroot = root = newnode();
	
	// make it white's move, so first move is black:
	root->who = 1;
	user = 0;
	playing = 1;
	autoplay=0;
	animating=-1;
	updateBoard();
	repaint();
}

char Board::check(int target,int direction) 
{
	// see if enemy at target can be jumped from direction by our piece
	int dst = target-direction;
	if (tb[dst]) 
		return(0);
	int src = target+direction;
	if (tb[src] == FRIENDKING);
	else if (direction < 0 || tb[src] != FRIEND) 
		return(0);
	piece a = tb[target]; piece b = tb[src];
	tb[target] = EMPTY; 
	tb[src] = EMPTY;
	int safe =
		(tb[src-4]&FRIEND && tb[src-8]&ENEMY
		||tb[src-5]&FRIEND && tb[src-10]&ENEMY
		||tb[dst-4]&ENEMY && !tb[dst+4]
		||tb[dst-5]&ENEMY && !tb[dst+5]
		||tb[src+4]&FRIEND && tb[src+8]==ENEMYKING
		||tb[src+5]&FRIEND && tb[src+10]==ENEMYKING
		||tb[dst+4]==ENEMYKING && !tb[dst-4]
		||tb[dst+5]==ENEMYKING && !tb[dst-5]);
	tb[target] = a; tb[src] = b;

	return(safe);
}

void Board::analyzemove(int direction,int src) 
{
	int target = src+direction;
	if (!tb[target]) 
	{
		if (!tb[target+direction])
			is_protected[target] = 1;
		piece a = tb[src]; tb[src] = EMPTY;
		if (check(target,4) || check(target,5) ||
				check(target,-4) || check(target,-5) ||
				(tb[src+4]&ENEMY && check(src+4,4)) ||
				(tb[src+5]&ENEMY && check(src+5,5)) ||
				(tb[src-4]&ENEMY && check(src-4,-4)) ||
				(tb[src-5]&ENEMY && check(src-5,-5)))
				deniedmoves++;
		else undeniedmoves++;
		tb[src] = a;
	}
}

void Board::evaluateboard(node *n,int print) 
{
		
	if (!n->who) 
		tb = b;	// move was black's
	else 
	{
		for (int i=0; i<45; i++) 
			flipboard[44-i] = flip[b[i]];
		tb = flipboard;
	}
		
	memset(is_protected,0,sizeof(is_protected));
	int friendpieces = 0;
	int enemypieces = 0;
	int friendkings = 0;
	int enemykings = 0;
	int friendkcent = 0;
	int friendcent = 0;
	int enemykcent = 0;
	int enemycent = 0;
	n->mobil = n->deny = n->pin = n->threat = 0;
		
	int i;
	for (i=5; i<40; i++) 
	{
		switch(tb[i]) 
		{
			case ENEMYKING:
				enemykings++;
				enemykcent += centralsquares[i];
				deniedmoves = 0;
				undeniedmoves = 0;
				if (i>8) 
				{
					analyzemove(-4,i);
					analyzemove(-5,i);
				}
				goto J1;
			case ENEMY:
				deniedmoves = 0;
				undeniedmoves = 0;
J1:				enemypieces++;
				enemycent += centralsquares[i];
				if (i<36) 
				{
					analyzemove(4,i);
					analyzemove(5,i);
				}
				if (deniedmoves && !undeniedmoves) 
					n->pin++;
				n->deny += deniedmoves;
				n->mobil += undeniedmoves;
				break;
			case FRIENDKING:
				friendkings++;
				friendkcent += centralsquares[i];
				if (tb[i+4]&ENEMY && !tb[i+8] && !(tb[i+4]==ENEMYKING && !tb[i-4]))
					n->threat++;
				if (tb[i+5]&ENEMY && !tb[i+10] && !(tb[i+5]==ENEMYKING && !tb[i-5]))
					n->threat++;
			case FRIEND:
				friendpieces++;
				friendcent += centralsquares[i];
				if (tb[i-4]&ENEMY && !tb[i-8] && tb[i+4]) 
					n->threat++;
				if (tb[i-5]&ENEMY && !tb[i-10] && tb[i+5]) 
					n->threat++;
				break;
		}
	}		
	
	int gradient[40];
	
	for (i=4; i<9; i++) 
		gradient[i] = tb[i] ? 0 : 32;
	int total = 0;
	for (i=9; i<40; i++) 
	{
		int x = (gradient[i-4]+gradient[i-5])/2;
		if (tb[i]==FRIEND) 
			total += x;
		gradient[i] = (tb[i]&FRIEND || !tb[i] && !is_protected[i]) ? x : 0;
	}
		
	n->gradient = total;
		
	n->back = tb[39]==FRIEND && tb[37]==FRIEND && !enemykings;
		
	node* f = n->father;
		
	n->moc2 = f->mobil>n->mobil && friendcent>enemycent;
	n->moc3 = f->mobil<=n->mobil && friendcent<enemycent;
	n->moc4 = f->mobil>n->mobil && friendcent<enemycent;
	n->mode2 = f->mobil<=n->mobil && n->deny<f->deny;
	n->mode3 = f->mobil>n->mobil && n->deny>f->deny;
	n->demmo = n->deny>f->deny && f->deny+f->mobil>n->deny+n->mobil;
		
	total =	spiece	* (friendpieces - enemypieces) +
			(sking-spiece) * (friendkings	- enemykings) +
			//	mobil?
			sdeny	* (n->deny	- f->deny) +
			spin	* (n->pin	- f->pin) +
			sthreat	* (n->threat	- f->threat) +
			sgrad	* (n->gradient	- f->gradient) +
			sback	* (n->back	- f->back) +
			smoc2	* (n->moc2	- f->moc2) +
			smoc3	* (n->moc3	- f->moc3) +
			smoc4	* (n->moc4	- f->moc4) +
			smode2	* (n->mode2	- f->mode2) +
			smode3	* (n->mode3	- f->mode3) +
			sdemmo	* (n->demmo	- f->demmo) +
			scent	* (friendcent	- enemycent) +
			(skcent-scent) * (friendkcent	- enemykcent);

	if (!n->mobil) 
		total += sallpin;
		
	if (!enemypieces) 
		total = 30000;
	else if (friendpieces > enemypieces)
		total += (sadvan*friendpieces)/enemypieces-sadvan;
	else 
		total -= (sadvan*enemypieces)/friendpieces-sadvan;
		
	if (print) 
	{
		printf("\tParent\tNew\tScore\n");
		printf("pieces\t%d\t%d\t%d\n",enemypieces,friendpieces,
			spiece*(friendpieces-enemypieces));
		printf("kings\t%d\t%d\t%d\n",enemykings,friendkings,
			(sking-spiece)*(friendkings-enemykings));
		printf("mobil\t%d\t%d\n",f->mobil,n->mobil);
		printf("deny\t%d\t%d\t%d\n",f->deny,n->deny,sdeny*(n->deny-f->deny));
		printf("pin\t%d\t%d\t%d\n",f->pin,n->pin,spin*(n->pin-f->pin));
		printf("threat\t%d\t%d\t%d\n",f->threat,n->threat,sthreat*(n->threat-f->threat));
		printf("grad\t%d\t%d\t%d\n",f->gradient,n->gradient,sgrad*(n->gradient-f->gradient));
		printf("back\t%d\t%d\t%d\n",f->back,n->back,sback*(n->back-f->back));
		printf("moc2\t%d\t%d\t%d\n",f->moc2,n->moc2,smoc2*(n->moc2-f->moc2));
		printf("moc3\t%d\t%d\t%d\n",f->moc3,n->moc3,smoc3*(n->moc3-f->moc3));
		printf("moc4\t%d\t%d\t%d\n",f->moc4,n->moc4,smoc4*(n->moc4-f->moc4));
		printf("mode2\t%d\t%d\t%d\n",f->mode2,n->mode2,smode2*(n->mode2-f->mode2));
		printf("mode3\t%d\t%d\t%d\n",f->mode3,n->mode3,smode3*(n->mode3-f->mode3));
		printf("demmo\t%d\t%d\t%d\n",f->demmo,n->demmo,sdemmo*(n->demmo-f->demmo));
		printf("cent\t%d\t%d\t%dn",enemycent,friendcent,scent*(friendcent-enemycent));
		printf("kcent\t%d\t%d\t%d\n",enemykcent,friendkcent,skcent*(friendkcent-enemykcent));
		printf("total:\t\t\t%d\n",total);
	}
	else 
	{
		n->value = total;
		evaluated++;
	}
}

void Board::movepiece(node* f, int i, node* jnode) 
{
	static char jumphappened;
	
	for (int k=0; k<4; k++) 
	{
		int direction = offset[b[i]][k];
		if (!direction) break;
		int j = i+direction;
		if (b[j] == EMPTY) 
		{
			if (!jnode && (!forcejumps || !f->son || !f->son->jump)) 
			{
				node* n = newnode();
				n->father = f;
				n->who = !f->who;
				n->from = i;
				n->to = j;
				piece oldpiece = b[i]; b[i] = EMPTY;
				if (!(oldpiece&KING) && n->who ? (j>=36) : (j<=8)) 
				{
					n->king = 1;
					b[j] = oldpiece|KING;
				}
				else 
					b[j] = oldpiece;
				evaluateboard(n,0);
				insert(n);
				b[i] = oldpiece; b[j] = EMPTY;
			}
		} else if (((b[j]^b[i])&(WHITE|BLACK))==(WHITE|BLACK) && !b[j+direction]) 
		{
			if (forcejumps && f->son && !f->son->jump) 
			{
				killnode(f->son);
				f->son = 0;
			}
			int jumploc = j;
			j += direction;
			node* n = newnode();
			n->father = f;
			n->who = !f->who;
			n->from = i;
			n->to = j;
			n->jump = (1<<(jumploc-10));
			piece oldpiece = b[i]; b[i] = EMPTY;
			if (!(oldpiece&KING) && n->who ? (j>=36) : (j<=8)) 
			{
				n->king = 1;
				b[j] = oldpiece|KING;
			}
			else 
				b[j] = oldpiece;
			if (jnode) 
			{
				n->from = jnode->from;
				n->jump |= jnode->jump;
				n->king |= jnode->king;
			}
			piece jumpedpiece = b[jumploc];
			b[jumploc] = EMPTY;
			jumphappened = 0;
			movepiece(f,j,n);
			if (forcejumps && jumphappened) 
				killnode(n);
			else 
			{
				evaluateboard(n,0); 
				insert(n);
			}
			b[i] = oldpiece; b[j] = EMPTY;
			b[jumploc] = jumpedpiece;
			jumphappened = 1;
		}
	}
}
void Board::expandnode(node *f) 
{
	if (f->son || f->value > 28000) 
		return;	// already done
	piece turn = f->who ? BLACK : WHITE;

	for (int i=5; i<40; i++)
	{
		if (b[i]&turn) 
			movepiece(f,i,0);
	}
	if (f->son) 
	{
		f->value = -f->son->value;
		if (f->brother) 
			f->value -= depthpenalty;
	}
	else 
		f->value = 30000;
}
void Board::makemove(node *n) 
{
	b[n->to] = b[n->from];
	if (n->king) 
		b[n->to] |= KING;
	b[n->from] = EMPTY;
	if (n->jump) 
	{
		for(int i=0; i<32; i++) 
		{
			if (n->jump & (1<<i)) 
				b[10+i] = EMPTY;
		}
	}
}
int Board::fullexpand(node *f, int level) 
{
	if (didabort() || nodes > maxnodes-(maxply*10) || evaluated > maxevaluate) 
		return(0);
	expandnode(f);
	
	if (!f->son) 
		return(1);
	
	piece oldboard[45];
	
	memmove(oldboard,b,sizeof(b));
	node* n = f->son;
	
	if (!n->jump && n->brother) 
	{
		if (level<1) 
			return(1); 
		level--;
	}
	int i;
	node* sons[32]; 
	for (i=0; (sons[i++] = n); n = n->brother);

	int ret = 1;
	for (i=0; ret && (n = sons[i++]);) 
	{
		makemove(n);
		ret = fullexpand(n,level);
		memmove(b,oldboard,sizeof(b));
		extract(n);
		insert(n);
	}
	f->value = -f->son->value;
	return(ret);
}

int Board::descend(node *f) 
{
	static int depth;
	if (didabort() || nodes > maxnodes || depth >= maxply) 
		return(0);
	if (f->son) 
	{
		node* n = f->son;
		makemove(n);
		depth++;
		int ret = descend(n);
		depth--;
		extract(n);
		insert(n);
		f->value = -f->son->value;
		return(ret);
	}
	else 
	{
		expandnode(f); 
		return(1);
	}
}
Board::node* Board::calcmove(Board::node *root) 
{	
	// return best move after root
	expandnode(root);
	
	if (!root->son) 
		return(0);	// no move due to loss

	evaluated = 0;
	if (root->son->brother) 
	{
		int x;
		for (x = 1; abs(root->value)<28000 && fullexpand(root,x); x++);

		piece saveboard[45]; memmove(saveboard,b,sizeof(b));
		while (abs(root->value)<28000) 
		{
			x = descend(root);
			memmove(b,saveboard,sizeof(b));
			if (!x) 
				break;
		}
	}
	return(root->son);
}

void Board::domove(node* move) 
{
	if (move->jump) 
		memmove(jumpboards[nextjump++],b,sizeof(b));
	makemove(move);
	extract(move);
	killnode(root->son);
	root->son = move;
	root = move;
	if (debug) 
		evaluateboard(move,1);
}
Board::node* Board::undomove() 
{
	node *n = root;
	if (n == undoroot) 
		return 0; // no more undo possible
	
	if (n->jump) 
		memmove(b,jumpboards[--nextjump],sizeof(b));
	else 
	{
		b[n->from] = b[n->to];
		if (n->king) 
			b[n->from] &= (WHITE|BLACK);
		b[n->to] = EMPTY;
	}
	root = n->father;
	killnode(n);
	root->son = 0;
	root->value = 0;	// prevent it from thinking game is over
	playing = 1;
	if (root == undoroot) 
		user = 0;
	return n;
}

Board::node *Board::newnode(void) 
{
	node *n;
	if (freelist) 
	{
		n = freelist;
		freelist = n->brother;
	}
	else 
		n = (node *)malloc(sizeof(node));
	memset(n,0,sizeof(node));
	nodes++;
	return(n);
}

void Board::extract(node *n) 
{
	node* i = n->father;
	if (i) 
	{
		node* j = i->son;
		if (j==n) 
			i->son = n->brother;
		else 
		{
			while (j) 
			{
				i = j; j = j->brother;
				if (j==n) 
				{
					i->brother = n->brother; 
					break;
				}
			}
		}
	}
	n->brother = 0;
}

void Board::killnode(node *x) 
{
	if (!x) 
		return;
	node *y;
	for (y = x; ; y = y->brother) 
	{
		nodes--;
		killnode(y->son); y->son = 0;
		if (!y->brother) 
			break;
	}
	y->brother = freelist;
	freelist = x;
}

void Board::insert(node *n) 
{
	int val = n->value;
	node **pp;
	for (pp = &(n->father->son); *pp; pp = &((*pp)->brother)) 
	{
		int val1 = (*pp)->value;
		if (abs(val-val1) <= noise) 
		{
			seed = (seed*13077+5051)%0100000;
			if ((seed & 070) >= 060) 
				break;
		}
		else if (val > val1) 
			break;
	}
	n->brother = *pp;
	*pp = n;
}




void Board::autoplay_cb() 
{
	if (!playing) 
		return;
	autoplay = 1;
	while (autoplay) 
	{
		computer_move(0); 
		updateBoard();
		repaint(); 
		computer_move(0);
		updateBoard();
		repaint(); 
	}
	if(playing)
	{
		emit playerMove();
		emit undoAvailable(true);
	}
}

bool Board::forced_cb() 
{
	forcejumps = !forcejumps;
	if(playing)
	{
		killnode(root->son); 
		root->son = 0;
		if(showlegal)
			expandnode(root); 
		selectedPiece=-1;
		updateBoard();
		repaint();
	}
	return forcejumps;
}

void Board::move_cb() 
{
	if (playing) 
	{
		computer_move(1);
		emit undoAvailable(true);
		updateBoard();
		repaint(); 
	}
	if (playing) 
	{
		computer_move(0);
		emit playerMove();
		emit undoAvailable(true);
		updateBoard();
		repaint(); 
	}
}

bool Board::legal_cb() 
{
	if (!playing) 
		return (bool) showlegal;

	if (showlegal == 1) 
	{
		showlegal = 0; 
		updateBoard();
		repaint();  
		return (bool)showlegal;
	}
	expandnode(root);
	showlegal = 1; 
	updateBoard();
	repaint(); 

	return (bool) showlegal;
}



void Board::switch_cb() 
{
	user = !user;
	computer_move(0);
	selectedPiece=-1;
	if(playing)
	{
		emit playerMove();
		emit undoAvailable(true);
	}
	updateBoard();
	repaint();
}

void Board::undo_cb() 
{
	if(animating==-1)
	{
		animate(undomove(),1);
		updateBoard();
		repaint();
		animate(undomove(),1);
		selectedPiece=-1;
		updateBoard();
		repaint();
		if(root==undoroot)
		{
			emit undoAvailable(false); // no more undo possible
		}
	}
}
void Board::stop_cb() 
{
	abortflag = 1;
}

void Board::stopAutoplay()
{
	autoplay=0;
	abortflag=0;
}

int Board::didabort(void) 
{
	qApp->processEvents(); 
	if(aquit)
		exit(0);					// nasty ... just drop everything and quit
	if (abortflag) 
	{
		abortflag = 0;
		return 1;
	}
	return(0);
}
