#ifndef BOARD_H
#define BOARD_H

#include <qwidget.h>
#include <qbitmap.h>
#include <qtimer.h>
#include <qpoint.h>
#include <qlist.h>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>
#include <time.h>


class ImageRepository;

class Board : public QWidget
{
	Q_OBJECT

public:
	Board(ImageRepository *img,int w, int h,QWidget* parent=0, const char* name=0, WFlags f=0 );




	void newgame(int ); 
	inline bool isPlaying()
	{
		return playing;
	}
	inline bool is_forced_cb()
	{
		return forcejumps;
	}
	// menu callback methods

	inline void quitApp()
	{
		aquit=true;
	}

	inline QPixmap* backPixmap()
	{
		return &back;
	}

	void autoplay_cb();
	bool isAutoplay()
	{
		return (bool)autoplay;
	}

	bool isAnimating()
	{
		if(animating==-1)
			return false;
		else 
			return true;
	}
	bool forced_cb();

	inline void setIntelligence(int i)
	{
		maxevaluate=maxnodes=i*(i*200);		
	}

	void move_cb() ;	
	bool legal_cb();
	inline bool is_legal_cb()
	{
		return (bool)showlegal;
	}
	void switch_cb();
	void undo_cb();
	void stop_cb();
	void stopAutoplay();

	bool isBoardSwitched()
	{
		return (bool)user;
	}

signals:

	void undoAvailable(bool );
	void playerMove();
	void computerMove();
	void gameOver(int loser);
	void computerResigned();

protected:
	void paintEvent ( QPaintEvent * );
	void mousePressEvent ( QMouseEvent * ); 
	void resizeEvent ( QResizeEvent * ); 

private slots:
	void synchTimeout();

private:

	void updateBoard(QPainter *p=0);

	int squarex(int i);
	int squarey(int i);

	void draw_piece(QPainter *p,int num,int which, int x, int y);
	void message(const char* m, ...); 
	static const char _usermoves[100]; 
	#define usermoves(x,y) _usermoves[2*((x)-5)+(y)-1]
	
struct node
	{
		struct node *father;
		struct node *son;					// best son
		struct node *brother;				// next brother
		short int value;					// value of this board position to player making move
		unsigned char from,to;				// the move to reach this board
		long int jump;						// bit map of locations jumped
		unsigned char mobil;
		unsigned char deny;
		unsigned char pin;
		unsigned char threat;
		short int gradient;
		unsigned who;	// 0 = black's move, 1 = white's move
		unsigned king;	// 1 = move causes piece to be kinged
		unsigned back;
		unsigned moc2;
		unsigned moc3;
		unsigned moc4;
		unsigned mode2;
		unsigned mode3;
		unsigned demmo;
};

//typedef struct node *NODEPTR;

// standard methods

	void init();
	void createHighlightedImage(QImage &img);

	node* selectPiece(int x, int y);
	bool movePiece(int x, int y);

	void drag_piece(int, int, int);
	void drop_piece(int);
	void animate(node* move, int backwards);
	void computer_move(int);


	
	int didabort(void);





// game methods



	char check(int target,int direction); 
	void analyzemove(int direction,int src);
	void evaluateboard(node *n,int print); 
	void movepiece(node* f, int i, node* jnode);
	void expandnode(node *f);
	void makemove(node *n);
	int fullexpand(node *f, int level);
	int descend(node *f); 
	node *calcmove(node *root); 
	void domove(node* move);
	node* undomove(); 



// tree management

	node *newnode(void) ;
	void extract(node *n); 
	void killnode(node *x);
	void insert(node *n);
	void dumpnode(node *n, int help);


	















	////////////////////////////////////////////////////////////////
	// The algorithim:
	
	int maxevaluate;		// max number of moves to examine on a turn
	int maxnodes;		// maximum number of nodes in search tree
	int maxply;		// maximum depth to look ahead
	bool forcejumps;		// is forced jumps rule in effect?
	
	// scoring parameters: (all divided by 5 from original code)
	// some signs seem to be backwards, marked them with (-) in comment
	int spiece;		// value of a piece
	int sking;		// value of a king
	int sadvan;		// value of mypieces/theirpieces-1
	// const int smobil = ?		// moves *enemy* can make w/o being jumped
	int sallpin;		// mobil == 0
	int sdeny;		// moves enemy can make that will be jumped
	int spin;		// enemy pieces that have no move except jumped
	int sthreat;	// enemy pieces we can jump if not moved (-)
	int sgrad;		// score of piece positions
	int sback;		// back row occupied so enemy can't make king
	int smoc2;		// more mobility, more center
	int smoc3;		// less mobility, less center
	int smoc4;		// more mobility, less center
	int smode2;		// less mobility, less denied
	int smode3;		// more mobility, more denied (-)
	int sdemmo;		// more denied, more moves (-)
	int scent;		// pieces in center
	int skcent;		// kings in center
	int depthpenalty;	// guess
	int noise;		// values less or eq to this apart are eq
	
	// const int sattackking = 4;	// not used
	// const int sattackpiece = 3;
	


typedef char piece;		

	bool aquit;
	int nodes;					// count of nodes
	int evaluated;				// number of moves evaluated this turn
	char centralsquares[45];
	char is_protected[45];
	piece flipboard[45];		// swapped if enemy is black
	piece *tb;					// pointer to real or swapped board




// Piece values so that BLACK and WHITE are bit flags:


#define EMPTY		0
#define BLACK		1
#define WHITE		2
#define KING		4
#define BLACKKING	5
#define WHITEKING	6
#define BLUE		8	

#define FRIEND			BLACK
#define FRIENDKING		BLACKKING
#define ENEMY			WHITE
#define ENEMYKING		WHITEKING

	static const char flip[9];
	static const int offset[9][4];
	piece b[45];		// current board position being considered

	int deniedmoves,undeniedmoves;
	node *freelist;
	int seed;		// current random number
	char debug;
	
	// the actual game state ----------------

	node *root,*undoroot;

	piece jumpboards[24][45];	// saved boards for undoing jumps
	int nextjump;

	char user;	// 0 = black, 1 = white
	char playing;
	char autoplay;



	int abortflag;
	int showlegal;


	QList<QPoint> animationList;
	QPixmap back;
	ImageRepository *images;
	QTimer timer;

	int busy; // causes pop-up abort menu
	static int erase_this;  // real location of dragging piece, don't draw it
	static int dragging;	// piece being dragged
	static int dragx;	// where it is
	static int dragy;
	int boxsize;
	int animating;
	bool drawNew;

	int selectedPiece;

	
};

#endif // BOARD_H
