#include "GameDialog.h"   
#include <qtimer.h>
#include <qrect.h>
#include <qpainter.h>
#include <stdlib.h>


GameDialog::GameDialog( const QString &text,int ofx, int ofy,QImage *gradient,QPixmap *background,QWidget *parent,const char *name):QDialog(parent,name,true,WStyle_Customize|WStyle_NoBorderEx)
{

	setBackgroundMode(NoBackground);
	if(parent)
		setFixedSize(gradient->width(),gradient->height());
	else
		setFixedSize(140,140);

	if(parent)
	{
		QRect pr=parent->frameGeometry ();
		move(pr.x()+(pr.width()-width())/2,pr.y()+((pr.height()-height())/2)+20);
	}

	QImage tmp=background->convertToImage();
	QRect r=parent->geometry(); 
	fback=tmp.copy(x()-r.x()-ofx,y()-r.y()-ofy,width(),height());	
	//lback.resize	
	final=fback.copy(0,0,fback.width(),fback.height());
	QPixmap	tpix(width()*2,height()*2);
	QPainter p(&tpix);
	QImage timg=gradient->smoothScale(tpix.width(),tpix.height());
	p.drawImage(0,0,timg);
	p.drawRect(0,0,tpix.width(),tpix.height());
	p.drawRect(2,2,tpix.width()-4,tpix.height()-4);
	QFont fnt=p.font();
	fnt.setPixelSize(height()-(height()/3));
	fnt.setBold(true);
	p.setFont(fnt);

	
	p.setPen(QColor(0,0,0));
	p.drawText(2,2,tpix.width(),tpix.height(),AlignCenter,text);	
	p.setPen(QColor(192,182,166).light(120));
	p.drawText(0,0,tpix.width(),tpix.height(),AlignCenter,text);
	lback=tpix.convertToImage();
	lback=lback.smoothScale(width(),height());

	timer = new QTimer(this);
	connect( timer, SIGNAL(timeout()),this, SLOT(timeSynch()));	
	dcounter=0;
	cstage=UP;
	timer->start(23);
}

void GameDialog::paintEvent ( QPaintEvent * )
{
	QPainter p(this);

	p.drawImage(0,0,final);
	
}

void GameDialog::mousePressEvent ( QMouseEvent * )
{
	if(cstage!=DOWN)
	{
		cstage=DOWN;
		dcounter=0.8;
	}
}

void GameDialog::keyPressEvent ( QKeyEvent * )
{
	if(cstage!=DOWN)
	{
		cstage=DOWN;
		dcounter=0.8;
	}
}

void GameDialog::timeSynch()
{	
	if(cstage==UP || DOWN)
	{
		final=fback.copy(0,0,width(),height());
		blendImage(lback, final,dcounter);
		repaint();
	}
	if(cstage==UP)
	{
		dcounter=dcounter+0.1;
		if(dcounter>=0.8)
		{
			wcounter=100;
			cstage=WAIT; 
		}
	}
	else if(cstage==DOWN)
	{
		dcounter=dcounter-0.1;
		if(dcounter<=0)
			accept();
	}
	else if(cstage==WAIT)
	{
		wcounter--;
		if(!wcounter)
			cstage=DOWN;
	}
}

void GameDialog::blendImage(QImage& src,QImage& dst, float opacity)
{
    unsigned int mult=(unsigned int)(opacity*(1<<8));

    if ( src.depth() != 32 ) src = src.convertDepth(32);
    if ( dst.depth() != 32 ) dst = dst.convertDepth(32);

    int pixels = src.width() * src.height();

    register unsigned char *ddata;
    register unsigned char *sdata;
	    
    int wsize;
    bool bendian;	    
	       
    qSysInfo(&wsize,&bendian);
    if(bendian)	    
    {	       
    	ddata = (unsigned char *)dst.bits() + 1;
    	sdata = (unsigned char *)src.bits() + 1;
    } else
    {                    
    	ddata = (unsigned char *)dst.bits();
    	sdata = (unsigned char *)src.bits();
    }

	
    for (register int i=0; i<pixels; i++)
    {
		*(ddata++)+=(unsigned char)((((unsigned int)(*(sdata++) - *ddata)<<8)*mult)>>16);
		*(ddata++)+=(unsigned char)((((unsigned int)(*(sdata++) - *ddata)<<8)*mult)>>16);
		*(ddata++)+=(unsigned char)((((unsigned int)(*(sdata++) - *ddata)<<8)*mult)>>16);
		ddata++; 
		sdata++;
    }

    return ;
}