/**********************************************************************
** Copyright (C) 2001 Walter Rawdanik.  All rights reserved.
**
** This file is part of Puzz-le game for Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** BSD license appearing in the file LICENSEL included in the
** packaging of this file.
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
**********************************************************************/

#include "ImageRepository.h"
#include "images.h"

static const char *imageNames[]= {
        "red_pion",
		"blue_pion",
		"red_queen",
		"blue_queen",
		"light_square",
		"dark_square",
		"title",
		"board256"
};

static const int imageProps[]= {1,1,1,1,0,0,0,0};

ImageRepository::ImageRepository()
{
	images.setAutoDelete(true);
}

ImageRepository::~ImageRepository()
{
	
}

QImage* ImageRepository::findImage(const QString& name)
{
    return images.find(name);
}

void ImageRepository::init(int sw, int sh)
{
	QImage *img;
	QString nName;
	
	int i;

	for(i=0;i<8;i++)
	{
		img=qembed_findImage(imageNames[i]);
		images.insert(imageNames[i],img);
		if(imageProps[i])
		{
			QImage *dark= new QImage(*img);
			dark->detach();
			createHighlighted(*dark);
			QString tmp(imageNames[i]);
			images.insert(tmp+"h",dark);
		}
	}
		
}

void ImageRepository::createHighlighted(QImage &img)
{
	int x,y;
	
	for(y=0;y<img.height();y++)
	{
		for(x=0;x<img.width();x++)
		{
			// Create 60% darker image
			// Seems like QColor does not retain alpha info which means we have to restore it later ...
			QRgb s=img.pixel(x,y);
			QColor clr(s);
			QRgb d=clr.dark(160).rgb();
			img.setPixel(x,y,qRgba(qRed(d),qGreen(d),qBlue(d),qAlpha(s)));         
		}
	}
	
}




