#include "checkerswindow.h"
#include <qpainter.h>
#include <qimage.h>
#include <qpe/qpemenubar.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qapplication.h>
#include "ImageRepository.h"
#include "GameDialog.h"




CheckersWindow::CheckersWindow( QWidget* parent, const char* name = 0, WFlags f )
	: QMainWindow( parent, name, f ),BoardWidth(224)
{
	int i;

	setCaption("Checkers");

	images=new ImageRepository();

	images->init(width(),height());
	
	board=new Board(images,8,8,this);
	board->setGeometry(0,0,224,224);

	setBackgroundMode(NoBackground);

	mb=new QPEMenuBar(this);

    game = new QPopupMenu;
    game->insertItem(tr("&New Game"), NEW_GAME);
	game->setAccel(CTRL+Key_N,NEW_GAME);
	game->insertSeparator();
	game->insertItem(tr("&Autoplay"), AUTOPLAY);
	game->insertItem(tr("&Switch Sides"), SWITCH_SIDES);
	game->insertSeparator();
	game->insertItem(tr("&Exit"), EXIT);
	mb->insertItem(tr("&Game"),game);
	connect( game, SIGNAL(activated(int)),SLOT(menuSelected(int))); 
	moves = new QPopupMenu;
    moves->insertItem(tr("&Undo Move"), UNDO_MOVE);
	moves->setAccel(CTRL+Key_Z,UNDO_MOVE);
	moves->insertSeparator();
	moves->insertItem(tr("&Move for Me"), MOVE_FOR_ME);
	moves->insertSeparator();
	moves->insertItem(tr("Stop &Thinking"), STOP_THINKING);
	moves->setAccel(Key_Escape,STOP_THINKING);
	mb->insertItem(tr("&Moves"),moves);
	connect( moves, SIGNAL(activated(int)),SLOT(menuSelected(int))); 
	options = new QPopupMenu;
	intelligence = new QPopupMenu;
    options->insertItem(tr("&Intelligence"), intelligence);
	for (i=0;i<10;i++)
	{
		intelligence->insertItem(QString::number(i+1), INTELLIGENCE+i+100);
	}
	intelligence->setItemChecked(INTELLIGENCE+105,true);
	board->setIntelligence(6);
	connect( intelligence, SIGNAL(activated(int)),SLOT(menuSelected(int))); 

	options->insertItem(tr("Forced &Jumps Rule"), FORCED_JUMPS_RULE);
	options->setItemChecked(FORCED_JUMPS_RULE,true);
	options->insertItem(tr("Show &Legal Moves"), SHOW_LEGAL_MOVES);
	options->setItemChecked(SHOW_LEGAL_MOVES,false);
	options->setAccel(CTRL+Key_L,SHOW_LEGAL_MOVES);
	mb->insertItem(tr("&Options"),options);
	connect( options, SIGNAL(activated(int)),SLOT(menuSelected(int))); 
	about = new QPopupMenu;
	about->insertItem(tr("&About the Game..."), ABOUT_THE_GAME);
	mb->insertItem(tr("&About"),about);

	okBtn=new QPushButton(tr("OK"),this);
	okBtn->hide();

	connect( about, SIGNAL(activated(int)),SLOT(menuSelected(int))); 
	connect( board, SIGNAL(undoAvailable(bool)),SLOT(undoAvailable(bool))); 
	connect( board, SIGNAL(computerMove()),SLOT(computerMove())); 
	connect( board, SIGNAL(playerMove()),SLOT(playerMove())); 
	connect( okBtn, SIGNAL(clicked()),SLOT(okButtonClicked()));
	connect( board, SIGNAL(gameOver(int)),SLOT(gameOver(int)));
	connect( board ,SIGNAL(computerResigned()),SLOT(computerResigned()));
	//connect(qApp, SIGNAL(lastWindowClosed() ),SLOT(lastWindowClosed() ));

	showAbout=false;	

	setIdle();
	//menuSelected(ABOUT_THE_GAME);

}

void CheckersWindow::paintEvent ( QPaintEvent * )
{
	QPainter p(this);
	QColor b(192,182,166);
	
	QPixmap tpix(width(),height());
	tpix.fill(QColor(192,182,166));
	QPainter tp(&tpix);
	
	if(!showAbout)
	{
		b=b.dark(200);
		tp.setPen(b);
		tp.drawRect(board->x()-1,board->y()-1,board->width()+2,board->height()+2);
		
		if(board->isPlaying())
		{
			int pixx=(width()-BoardWidth)/2;
			int pixy=board->y()-22-4;
			tp.setPen(QColor(0,0,0));		
		
			QString cmove("");
			
			if(isComputerMove) {
				cmove=tr("Thinking ...");
			} else if(board->isBoardSwitched()) {
				cmove=tr("Your turn to move (red)");
			} else {
				cmove=tr("Your turn to move (blue)");
			}
			
			tp.drawText(pixx,pixy+4,width(),height(),AlignLeft|AlignTop,cmove);
		}
	}
	else
	{
		QImage *tmp1=images->findImage("title");
		tp.drawImage((width()-tmp1->width()+2)/2,5,*tmp1);
		QImage *tmp2=images->findImage("board256");
		tp.drawImage((width()-tmp2->width())/2,okBtn->y()-tmp2->height()+8,*tmp2);
		
		
		QString cop(tr("Original Pascal code by\n Steve Poulsen 18-Jan-79\n C version written by Bill Spitzak\nPorted to Qtopia by Walter Rawdanik\nhttp://www.warmi.net/zaurus\nAdapted by Robert Ernst\nhttp://www.linux-solutions.at/projects/zaurus\nThis program is licensed under GPL."));

		QFont f=tp.font();
		f.setBold(true);
		tp.setFont(f);
		tp.setPen(QColor(192,182,166).light(120));
		tp.drawText(0+1,tmp1->height()-2+1,width(),1024, AlignHCenter|AlignTop,cop);
		tp.setPen(QColor(0,0,0));
		tp.drawText(0,tmp1->height()-2,width(),1024, AlignHCenter|AlignTop,cop);

	}
	p.drawPixmap(0,0,tpix);
}

void CheckersWindow::resizeEvent ( QResizeEvent * )
{
	int spacing=(width()-BoardWidth)/2;
	board->setGeometry(spacing,height()-BoardWidth-spacing,BoardWidth,BoardWidth);
	okBtn->setGeometry((width()-60)/2,height()-24,60,22);
}

void CheckersWindow::createGradientImage(QImage *img)
{
		
	float opacity=0;
	float wc=1.0/img->width();

	int i;
	int j;

	unsigned char dr;
	unsigned char dg;
	unsigned char db;

	for(i=0;i<img->width();i++)
	{
		unsigned int mult=opacity*(1<<8);
		opacity+=wc;
		dr=(unsigned char)((((unsigned int)192 <<8)*mult)>>16);
		dg=(unsigned char)((((unsigned int)182 <<8)*mult)>>16);
		db=(unsigned char)((((unsigned int)166 <<8)*mult)>>16);
		for (j=0; j<img->height(); j++)
		{
			img->setPixel(i,j,qRgb(dr,dg,db));
		}
	}


}

void CheckersWindow::menuSelected(int id)
{
	if(board->isAnimating())
	{
		if((id!=AUTOPLAY && id!=EXIT) || (id==AUTOPLAY && !board->isAutoplay()))
		{
			return;
		}
	}
	
	switch(id)
	{
	case	NEW_GAME:
		startNewGame();
		break;
	case	SWITCH_SIDES:
		board->switch_cb();
		repaint();
		break;
	case	UNDO_MOVE:
		board->undo_cb();
		repaint();
	case	FORCED_JUMPS_RULE:
		options->setItemChecked(FORCED_JUMPS_RULE,board->forced_cb());
		break;
	case	SHOW_LEGAL_MOVES:
		options->setItemChecked(SHOW_LEGAL_MOVES,board->legal_cb());
		break;
	case	MOVE_FOR_ME:
		board->move_cb();
		break;
	case	ABOUT_THE_GAME:
		board->hide();
		showAbout=true;
		okBtn->show();
		mb->hide();
		repaint();
		break;
	case	STOP_THINKING:
		board->stop_cb();
		repaint();
		break;
	case	AUTOPLAY:
		if(board->isAutoplay())
		{
			board->stopAutoplay();
			board->repaint();
			playerMove();
			game->changeItem(AUTOPLAY,tr("Autoplay"));
		}
		else
		{
			game->changeItem(AUTOPLAY,tr("Stop Autoplay"));
			board->autoplay_cb();
			
		}
		break;
		
	case	EXIT:
		close();
		break;
	case	INTELLIGENCE+100:
	case	INTELLIGENCE+101:
	case	INTELLIGENCE+102:
	case	INTELLIGENCE+103:
	case	INTELLIGENCE+104:
	case	INTELLIGENCE+105:
	case	INTELLIGENCE+106:
	case	INTELLIGENCE+107:		
	case	INTELLIGENCE+108:
	case	INTELLIGENCE+109:
		int il=id-INTELLIGENCE-100+1;
		board->setIntelligence(id-INTELLIGENCE-100+1);
		for(int i=0;i<10;i++)
		{
			intelligence->setItemChecked(INTELLIGENCE+i+100,false);
		}
		intelligence->setItemChecked(id,true);
		break;
	}
}

void CheckersWindow::startNewGame()
{

	board->newgame(1);
	game->setItemEnabled(SWITCH_SIDES,true);
	game->setItemEnabled(AUTOPLAY,true);
	moves->setItemEnabled(UNDO_MOVE,true);
	moves->setItemEnabled(MOVE_FOR_ME,true);
	isComputerMove=false;
	undoAvailable(false);
	repaint();
}

void CheckersWindow::setIdle()
{
	game->setItemEnabled(NEW_GAME,true);
	game->setItemEnabled(AUTOPLAY,false);
	game->setItemEnabled(SWITCH_SIDES,false);
	moves->setItemEnabled(UNDO_MOVE,false);
	moves->setItemEnabled(MOVE_FOR_ME,false);
	moves->setItemEnabled(STOP_THINKING,false);
	moves->setItemEnabled(STOP_THINKING,false);
	options->setItemEnabled(INTELLIGENCE,true);
	options->setItemEnabled(FORCED_JUMPS_RULE,true);
	options->setItemEnabled(SHOW_LEGAL_MOVES,true);
	about->setItemEnabled(ABOUT_THE_GAME,true);
}

void CheckersWindow::gameEnded(int loser)
{
	setIdle();
}

void CheckersWindow::undoAvailable(bool f)
{
	moves->setItemEnabled(UNDO_MOVE,f);	
}

void CheckersWindow::computerMove()
{
	game->setItemEnabled(NEW_GAME,false);
	game->setItemEnabled(AUTOPLAY,board->isAutoplay());
	game->setItemEnabled(SWITCH_SIDES,false);
	moves->setItemEnabled(UNDO_MOVE,false);
	moves->setItemEnabled(MOVE_FOR_ME,false);
	moves->setItemEnabled(STOP_THINKING,!board->isAutoplay());
	options->setItemEnabled(INTELLIGENCE,false);
	options->setItemEnabled(FORCED_JUMPS_RULE,false);
	options->setItemEnabled(SHOW_LEGAL_MOVES,false);
	about->setItemEnabled(ABOUT_THE_GAME,!board->isAutoplay());
	isComputerMove=true;
	repaint();
}

void CheckersWindow::playerMove()
{
	game->setItemEnabled(NEW_GAME,true);	
	game->setItemEnabled(AUTOPLAY,true);
	game->setItemEnabled(SWITCH_SIDES,true);
	moves->setItemEnabled(MOVE_FOR_ME,true);
	moves->setItemEnabled(STOP_THINKING,false);
	options->setItemEnabled(INTELLIGENCE,true);
	options->setItemEnabled(FORCED_JUMPS_RULE,true);
	options->setItemEnabled(SHOW_LEGAL_MOVES,true);
	about->setItemEnabled(ABOUT_THE_GAME,true);
	isComputerMove=false;
	repaint();
}

void CheckersWindow::okButtonClicked()
{
	board->show();
	showAbout=false;
	okBtn->hide();
	mb->show();
	repaint();
}

void CheckersWindow::gameOver(int loser)
{
	if ((loser == WHITE && !board->isBoardSwitched())
		|| (loser != WHITE && board->isBoardSwitched())) {
		displayMsg(tr("You won!"));
	} else {
		displayMsg(tr("Computer won!"));
	}
	repaint();
	setIdle();
	
}

void CheckersWindow::computerResigned()
{
	displayMsg(tr("Computer resigned"));
	repaint();
	setIdle();
}

void CheckersWindow::closeEvent ( QCloseEvent *e )
{
	board->stopAutoplay();
	board->quitApp();
	e->accept();
}

void CheckersWindow::displayMsg(const QString &msg)
{
	QImage img(width()-30,height()/5,32);
	createGradientImage(&img);
	GameDialog *gdlg=new GameDialog(msg,board->x(),board->y(),&img,board->backPixmap(),this);
	gdlg->exec();
}
