#ifndef CHECKERS_WINDOW_H
#define CHECKERS_WINDOW_H
#include <qpe/qpemenubar.h>
#include <qmainwindow.h>
#include "Board.h"
#include <qpixmap.h>

class ImageRepository;
class QPopupMenu;
class QPushButton;


class CheckersWindow : public QMainWindow
{
	Q_OBJECT
public:
	CheckersWindow( QWidget* parent = 0, const char* name = 0, WFlags f = 0 );

public slots:

	void undoAvailable(bool f);
	void gameEnded(int loser);
	void computerMove();
	void playerMove();

protected:
	void paintEvent ( QPaintEvent * );
	void resizeEvent ( QResizeEvent * ); 
	void closeEvent ( QCloseEvent * ) ;

	
private slots:

	void menuSelected(int);
	void okButtonClicked();
	void gameOver(int loser);
	void computerResigned();
	void setIdle();
	
private:

	enum MenuItems
	{
		NEW_GAME,
		AUTOPLAY,
		STOP_AUTOPLAY,
		SWITCH_SIDES,
		EXIT,
		UNDO_MOVE,
		MOVE_FOR_ME,
		STOP_THINKING,
		INTELLIGENCE,
		FORCED_JUMPS_RULE,
		SHOW_LEGAL_MOVES,
		ABOUT_THE_GAME

	};

	void createTopBarImages(int w, int h);
	void createGradientImage(QImage *img);
	void displayMsg(const QString &msg);
	
	void startNewGame();

	const int BoardWidth;

	ImageRepository *images;
	Board *board;
	
	QPopupMenu* game;
	QPopupMenu* moves;
	QPopupMenu* options;
	QPopupMenu* intelligence;
	QPopupMenu* about;
	QPushButton *okBtn;
	bool isComputerMove;
	bool showAbout;

	QPEMenuBar *mb;
};
#endif // CHECKERS_WINDOW_H
